function [Fint,K] = fuerza_y_matriz_elemental(u,Coord,Material)
% [Fint,K] = fuerza_y_matriz_elemental(u,Coord,Material)
% Funcion q calcula el vector de fuerzzs internas Fint y la
% matriz tangente elemental K para un elemento finito
% hexaedrico trilineal (hexaedro de 8 nodos ubicados en los vertices)
% Entradas:
% u = [uX1 uX2 uX3 uY1 uY2 uY3 uZ1 uZ2 uZ3 u41 u42 u43... u83]'
% uij: desplazamiento del nodo i en la direccin j
% Coord(i,j): coordenada j del nodo i
% Salidas:
% Fint(i): componente i del vector de fuerzas internas elemental
% K(i,j) = dFint(i)/du(j)


ux = u(1:3:24);  % Desplazamientos nodales en la direccin 1
uy = u(2:3:24);  % Desplazamientos nodales en la direccin 2
uz = u(3:3:24);  % Desplazamientos nodales en la direccin 3

% Inicializacin de matrices
B = zeros(6,24);
K = zeros(24,24);
KG = zeros(24,24);
Fint = zeros(24,1);

% Loop sobre los puntos de integracion numerica
NUMPI = 8;
for i = 1:NUMPI
    % Coordenadas naturales y peso del punto de Gauss i
    [wi,xi,eta,zeta] = fillWeightAndCoordHex(i,NUMPI);
    N = fillValFF3q1(xi, eta, zeta);
    % jacobiano de transformacin de coordenadas y
    % derivadas de las funciones de forma con respecto a las coordenadas
    % naturales
    
    [J, DN_Dxi, DN_Deta, DN_Dzeta] = fillJacoHexLin(xi,eta,zeta,Coord);
    detJ = det(J);
    if detJ<=0
        error('Determinante negativo o nulo')
    end
    J_inv = inv(J);
    
    % Derivadas de las funciones de forma con respecto a las coordenadas
    % globales    
    DN_DX = J_inv(1,1) * DN_Dxi + J_inv(1,2) * DN_Deta + J_inv(1,3) * DN_Dzeta;
    DN_DY = J_inv(2,1) * DN_Dxi + J_inv(2,2) * DN_Deta + J_inv(2,3) * DN_Dzeta;
    DN_DZ = J_inv(3,1) * DN_Dxi + J_inv(3,2) * DN_Deta + J_inv(3,3) * DN_Dzeta;
    
    % Gradiente de desplazamientos
    D(1,1) = dot(DN_DX,ux);
    D(2,1) = dot(DN_DX,uy);
    D(3,1) = dot(DN_DX,uz);
    D(1,2) = dot(DN_DY,ux);
    D(2,2) = dot(DN_DY,uy);
    D(3,2) = dot(DN_DY,uz);
    D(1,3) = dot(DN_DZ,ux);
    D(2,3) = dot(DN_DZ,uy);
    D(3,3) = dot(DN_DZ,uz);
    % Gradiente de deformacin
    A = D+eye(3);
    % Tensor de deformacin de Green-Lagrange
    Etensor = 0.5*(A'*A-eye(3));
    % Tensor de deformacin de Green-Lagrange en notacin de Voigt
    Evector = [Etensor(1,1); Etensor(2,2); Etensor(3,3); 2*Etensor(1,2); 2*Etensor(2,3); 2*Etensor(3,1)];
    % Clculo del 2o tensor de tensin de Piola-Kirchhoff
    % y del tensor de 4o orden de mdulos tangentes
    % (dados en notacin de Voigt) invocando la correspondiente
    % ley constitutiva
    [Tvector,Cmatriz] = tension_y_modulo_tangente(Evector,Material);
    Ttensor(1,1) = Tvector(1);
    Ttensor(1,2) = Tvector(4);
    Ttensor(1,3) = Tvector(6);
    Ttensor(2,1) = Ttensor(1,2);
    Ttensor(2,2) = Tvector(2);
    Ttensor(2,3) = Tvector(5);
    Ttensor(3,1) = Ttensor(1,3);
    Ttensor(3,2) = Ttensor(2,3);
    Ttensor(3,3) = Tvector(3);
    % Clculo de las matrices B y KG
    for I = 1:8
        BI = [
            DN_DX(I)*A(1,1), DN_DX(I)*A(2,1), DN_DX(I)*A(3,1)
            DN_DY(I)*A(1,2), DN_DY(I)*A(2,2), DN_DY(I)*A(3,2)
            DN_DZ(I)*A(1,3), DN_DZ(I)*A(2,3), DN_DZ(I)*A(3,3)
            DN_DY(I)*A(1,1)+DN_DX(I)*A(1,2), DN_DY(I)*A(2,1)+DN_DX(I)*A(2,2), DN_DY(I)*A(3,1)+DN_DX(I)*A(3,2)
            DN_DZ(I)*A(1,2)+DN_DY(I)*A(1,3), DN_DZ(I)*A(2,2)+DN_DY(I)*A(2,3), DN_DZ(I)*A(3,2)+DN_DY(I)*A(3,3)
            DN_DX(I)*A(1,3)+DN_DZ(I)*A(1,1), DN_DX(I)*A(2,3)+DN_DZ(I)*A(2,1), DN_DX(I)*A(3,3)+DN_DZ(I)*A(3,1)
            ];
        B(:,3*I-2:3*I) = BI;
        for J = 1:8
            GIJ = ...
                DN_DX(I)*Ttensor(1,1)*DN_DX(J)+DN_DX(I)*Ttensor(1,2)*DN_DY(J)+DN_DX(I)*Ttensor(1,3)*DN_DZ(J)+...
                DN_DY(I)*Ttensor(2,1)*DN_DX(J)+DN_DY(I)*Ttensor(2,2)*DN_DY(J)+DN_DY(I)*Ttensor(2,3)*DN_DZ(J)+...
                DN_DZ(I)*Ttensor(3,1)*DN_DX(J)+DN_DZ(I)*Ttensor(3,2)*DN_DY(J)+DN_DZ(I)*Ttensor(3,3)*DN_DZ(J);
            KG(3*I-2:3*I,3*J-2:3*J) = GIJ*eye(3);
        end
    end
    % Suma de la contribucin del punto de Gauss
    dw = detJ*wi;
    K = K + (B'*Cmatriz*B + KG)*dw;
    Fint = Fint + B'*(Tvector*dw);
end


